package javax.swing.text;

import java.util.Enumeration;
import java.util.Stack;

public class ElementIterator implements Cloneable
{
    private Element root;
    private Stack<ElementIterator.StackItem> elementStack;
    
    public ElementIterator(Document document) {
        this.elementStack = null;
        this.root = document.getDefaultRootElement();
    }
    
    public ElementIterator(Element root) {
        this.elementStack = null;
        this.root = root;
    }
    
    public synchronized Object clone() {
        try {
            ElementIterator elementIterator = new ElementIterator(this.root);
            if (this.elementStack != null) {
                elementIterator.elementStack = new Stack();
                for (int i = 0; i < this.elementStack.size(); ++i) {
                    elementIterator.elementStack.push((ElementIterator.StackItem)((ElementIterator.StackItem)this.elementStack.elementAt(i)).clone());
                }
                return elementIterator;
            }
            return elementIterator;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    public Element first() {
        if (this.root == null) {
            return null;
        }
        this.elementStack = new Stack();
        if (this.root.getElementCount() != 0) {
            this.elementStack.push(new ElementIterator.StackItem(this.root));
        }
        return this.root;
    }
    
    public int depth() {
        if (this.elementStack == null) {
            return 0;
        }
        return this.elementStack.size();
    }
    
    public Element current() {
        if (this.elementStack == null) {
            return this.first();
        }
        if (!this.elementStack.empty()) {
            ElementIterator.StackItem stackItem = (ElementIterator.StackItem)this.elementStack.peek();
            Element access$100 = ElementIterator.StackItem.access$100(stackItem);
            int access$101 = ElementIterator.StackItem.access$200(stackItem);
            if (access$101 == -1) {
                return access$100;
            }
            return access$100.getElement(access$101);
        }
        else {
            return null;
        }
    }
    
    public Element next() {
        if (this.elementStack == null) {
            return this.first();
        }
        if (this.elementStack.isEmpty()) {
            return null;
        }
        ElementIterator.StackItem stackItem = (ElementIterator.StackItem)this.elementStack.peek();
        Element access$100 = ElementIterator.StackItem.access$100(stackItem);
        int access$101 = ElementIterator.StackItem.access$200(stackItem);
        if (access$101 + 1 < access$100.getElementCount()) {
            Element element = access$100.getElement(access$101 + 1);
            if (element.isLeaf()) {
                ElementIterator.StackItem.access$300(stackItem);
                return element;
            }
            this.elementStack.push(new ElementIterator.StackItem(element));
            return element;
        }
        else {
            this.elementStack.pop();
            if (!this.elementStack.isEmpty()) {
                ElementIterator.StackItem.access$300((ElementIterator.StackItem)this.elementStack.peek());
                return this.next();
            }
            return null;
        }
    }
    
    public Element previous() {
        int size;
        if (this.elementStack == null || (size = this.elementStack.size()) == 0) {
            return null;
        }
        ElementIterator.StackItem stackItem = (ElementIterator.StackItem)this.elementStack.peek();
        Element access$100 = ElementIterator.StackItem.access$100(stackItem);
        int access$101 = ElementIterator.StackItem.access$200(stackItem);
        if (access$101 > 0) {
            return this.getDeepestLeaf(access$100.getElement(--access$101));
        }
        if (access$101 == 0) {
            return access$100;
        }
        if (access$101 == -1) {
            if (size == 1) {
                return null;
            }
            ElementIterator.StackItem item = (ElementIterator.StackItem)this.elementStack.pop();
            ElementIterator.StackItem stackItem2 = (ElementIterator.StackItem)this.elementStack.peek();
            this.elementStack.push(item);
            Element access$102 = ElementIterator.StackItem.access$100(stackItem2);
            int access$103 = ElementIterator.StackItem.access$200(stackItem2);
            return (access$103 == -1) ? access$102 : this.getDeepestLeaf(access$102.getElement(access$103));
        }
        else {
            return null;
        }
    }
    
    private Element getDeepestLeaf(Element element) {
        if (element.isLeaf()) {
            return element;
        }
        int elementCount = element.getElementCount();
        if (elementCount == 0) {
            return element;
        }
        return this.getDeepestLeaf(element.getElement(elementCount - 1));
    }
    
    private void dumpTree() {
        Element next;
        while ((next = this.next()) != null) {
            System.out.println("elem: " + next.getName());
            AttributeSet attributes = next.getAttributes();
            String str = "";
            Enumeration<Object> attributeNames = attributes.getAttributeNames();
            while (attributeNames.hasMoreElements()) {
                Object nextElement = attributeNames.nextElement();
                Object attribute = attributes.getAttribute(nextElement);
                if ((attribute instanceof AttributeSet)) {
                    str = str + nextElement + "=**AttributeSet** ";
                }
                else {
                    str = str + nextElement + "=" + attribute + " ";
                }
            }
            System.out.println("attributes: " + str);
        }
    }
    
    private class StackItem implements Cloneable
    {
        Element item;
        int childIndex;
        
        private StackItem(Element item) {
            this.item = item;
            this.childIndex = -1;
        }
        
        private void incrementIndex() {
            ++this.childIndex;
        }
        
        private Element getElement() {
            return this.item;
        }
        
        private int getIndex() {
            return this.childIndex;
        }
        
        @Override
        protected Object clone() throws CloneNotSupportedException {
            return super.clone();
        }
        
        static Element access$100(ElementIterator.StackItem stackItem) {
            return stackItem.getElement();
        }
        
        static int access$200(ElementIterator.StackItem stackItem) {
            return stackItem.getIndex();
        }
        
        static void access$300(ElementIterator.StackItem stackItem) {
            stackItem.incrementIndex();
        }
    }
}
